@@ -142,10 +142,6 @@
 # endif
 #endif
 
-#ifndef HAVE_MKTIME
-extern time_t mktime(struct tm *tp);
-#endif
-
 #ifndef FNM_CASEFOLD
 # define FNM_CASEFOLD (0)
 #endif
@@ -206,7 +202,7 @@ struct cabextract_args args = {
   NULL, NULL, NULL
 };
 
-#ifdef HAVE_ICONV
+#if HAVE_ICONV
 iconv_t converter = NULL;
 #endif
 
@@ -328,12 +324,12 @@ int main(int argc, char *argv[]) {
       "  -t   --test        test cabinet integrity\n"
       "  -q   --quiet       only print errors and warnings\n"
       "  -L   --lowercase   make filenames lowercase\n"
-      "  -f   --fix         fix (some) corrupted cabinets\n");
+      "  -f   --fix         salvage as much as possible from corrupted cabinets\n");
     fprintf(stderr,
       "  -p   --pipe        pipe extracted files to stdout\n"
       "  -s   --single      restrict search to cabs on the command line\n"
       "  -F   --filter      extract only files that match the given pattern\n"
-#ifdef HAVE_ICONV
+#if HAVE_ICONV
       "  -e   --encoding    assume non-UTF8 filenames have the given encoding\n"
 #endif
       "  -d   --directory   extract all files to the given directory\n\n"
@@ -396,10 +392,11 @@ int main(int argc, char *argv[]) {
   umask(user_umask = umask(0));
 #endif
 
-  /* turn on/off 'fix MSZIP' mode */
+  /* turn on/off 'fix MSZIP' and 'salvage' mode */
   cabd->set_param(cabd, MSCABD_PARAM_FIXMSZIP, args.fix);
+  cabd->set_param(cabd, MSCABD_PARAM_SALVAGE, args.fix);
 
-#ifdef HAVE_ICONV
+#if HAVE_ICONV
   /* set up converter for given encoding */
     if (args.encoding) {
       if ((converter = iconv_open("UTF8", args.encoding)) == (iconv_t) -1) {
@@ -876,7 +873,7 @@ static char *create_output_name(const char *fname, const char *dir,
         x = 0xFFFD; /* invalid code point or cheeky null byte */
       }
 
-#ifdef HAVE_TOWLOWER
+#if HAVE_TOWLOWER
       if (lower) x = towlower(x);
 #else
       if (lower && x < 256) x = tolower(x);
@@ -1169,10 +1166,13 @@ static int ensure_filepath(char *path) {
 static char *cab_error(struct mscab_decompressor *cd) {
   switch (cd->last_error(cd)) {
   case MSPACK_ERR_OPEN:
+    return errno ? strerror(errno) : "file open error";
   case MSPACK_ERR_READ:
+    return errno ? strerror(errno) : "file read error";
   case MSPACK_ERR_WRITE:
+    return errno ? strerror(errno) : "file write error";
   case MSPACK_ERR_SEEK:
-    return strerror(errno);
+    return errno ? strerror(errno) : "file seek error";
   case MSPACK_ERR_NOMEMORY:
     return "out of memory";
   case MSPACK_ERR_SIGNATURE:
